#!/usr/bin/env python

#####################################################
# Name: loadEngines.py
# Author: Amit Sharma
# Description: Startup program that starts an engine.
#   uses dbus client side
#
#####################################################

import sys, time
from traceback import print_exc

import dbus
import dbus.mainloop.glib
import gobject

global remote_object
global bus
global iface

# define tuples in the list of engines. tuple is (enginename,delay)
#enginelist = [('Everybody',5),('Oscope',5),('DMM',5),('UUTPS_Factory',5),('Scripting',5),('DSPEngine',5),('SpecAn',5),('RCVR',5),('RFGen',5),('NFL',5),('System,5'),('GUI',1)]
#enginelist = [('HardwareEnum',5),('Everybody',5),('DSPEngine',5),('Oscope',5),('DMM',5),('UUTPS_Factory',5),('Scripting',5),('System',5)]
enginelist = [('HardwareEnum',1)]

class serviceClient:

  def __init__(self):
    dbus.mainloop.glib.DBusGMainLoop(set_as_default=True)
    self.bus = dbus.SessionBus()
    try:
        self.remote_object = self.bus.get_object("CommonPlatform.Engine.ServiceInterface","/CommonPlatform/Engine/ServiceInterface/Object")

#        self.remote_object.connect_to_signal("DoneSignal",self.Done_signal_handler,dbus_interface="CommonPlatform.Engine.ServiceInterface", arg0= "Done")
    except dbus.DBusException:
        print_exc()
        #print usage
        sys.exit(1)
    # ... or create an Interface wrapper for the remote object
    self.iface = dbus.Interface(self.remote_object, "CommonPlatform.Engine.ServiceInterface")
    
    self.bus.add_signal_receiver(self.Done_signal_handler, dbus_interface = "CommonPlatform.Engine.ServiceInterface", signal_name = "DoneSignal")
    self.bus.add_signal_receiver(self.Start_signal_handler, dbus_interface = "CommonPlatform.Engine.ServiceInterface", signal_name = "StartSignal")
    self.bus.add_signal_receiver(self.Status_signal_handler, dbus_interface = "CommonPlatform.Engine.ServiceInterface", signal_name = "StatusSignal")

  def emitsignal(self,typeval,strparam):
    if(typeval == 0):
      self.iface.emitStartSignal(strparam)
    if(typeval == 1):
      self.iface.emitStatusSignal(strparam)
    if(typeval == 2):
      self.remote_object.emitDoneSignal(dbus_interface="CommonPlatform.Engine.ServiceInterface")


  def Done_signal_handler(self,instring):
    print "Got Done signal : " + instring

  def Start_signal_handler(self,instring):
    print "Got Start signal : " + instring

  def Status_signal_handler(self,instring):
    print "Got Status signal : " + instring

  def  RunServicesGlobal(self):
    val = self.iface.Start("Naming")
    time.sleep(1)
    val = self.iface.Start("DDS")
    time.sleep(2)


  def  RunServices(self):
    startlist = ""
    for iv, delayval in enginelist:
      if (len(startlist) != 0):
       startlist = startlist + ',' + str(iv)
      else:
       startlist = startlist + str(iv)
    val = self.iface.Start("Naming")
    time.sleep(1)
    val = self.iface.Start("DDS")
    time.sleep(2)
    for ival , delay in enginelist:
      print "Starting - " + str(ival) + " delay " + str(delay)
      val = self.iface.Start(ival)
      if(str(ival) == 'GUI'):
        print "Found GUI"
        time.sleep(1)
        self.emitsignal(0,startlist)
        time.sleep(delay)
        self.emitsignal(1,str(ival))
      else:       
        time.sleep(delay)
        self.emitsignal(1,str(ival))

  def  RunServicesLocal(self):
    startlist = ""
    for iv, delayval in enginelist:
      if (len(startlist) != 0):
       startlist = startlist + ',' + str(iv)
      else:
       startlist = startlist + str(iv)
    self.emitsignal(0,startlist)
    time.sleep(1)
    for ival , delay in enginelist:
      print "Starting - " + str(ival) + " delay " + str(delay)
      val = self.iface.Start(ival)
      if(str(ival) == 'GUI'):
        print "Found GUI"
        time.sleep(1)
        self.emitsignal(0,startlist)
        time.sleep(delay)
        self.emitsignal(1,str(ival))
      else:       
        time.sleep(delay)
        self.emitsignal(1,str(ival))

  def  StopServices(self):
    for ival, delay in enginelist:
      val = self.iface.Stop(ival)
    val = self.iface.Stop("Naming")
    val = self.iface.Stop("DDS")

  def  StopServicesGlobal(self):
    val = self.iface.Stop("Naming")
    val = self.iface.Stop("DDS")

  def  StopServicesLocal(self):
    for ival, delay in enginelist:
      val = self.iface.Stop(ival)

  def  Kill(self):
    self.iface.Kill()

def main():
    if sys.argv[1:] == ['--exit']:
      sc = serviceClient()
      sc.StopServices()
      sc.Kill()
    elif sys.argv[1:] == ['--local']:
      sc = serviceClient()
      sc.RunServicesLocal()
      sc.emitsignal(2,"")
    elif sys.argv[1:] == ['--global']:
      sc = serviceClient()
      sc.RunServicesGlobal()
      sc.emitsignal(2,"")
    elif sys.argv[1:] == ['--start']:
      sc = serviceClient()
      sc.RunServices()
      sc.emitsignal(2,"")
    elif sys.argv[1:] == ['--stop']:
      sc = serviceClient()
      sc.StopServices()
    elif sys.argv[1:] == ['--stopglobal']:
      sc = serviceClient()
      sc.StopServicesGlobal()
    elif sys.argv[1:] == ['--stoplocal']:
      sc = serviceClient()
      sc.StopServicesLocal()

    else:
      print "Usgae : ./loadEngines.py --start | --stop | --exit | --global | --local | --stopglobal | --stoplocal"
      print "      : --start  - loads global and local engines"
      print "      : --stop   - stops all the services"
      print "      : --exit   - kills all the services"
      print "      : --global - loads DDS and Nameservice"
      print "      : --local  - loads the engines in the tuple list enginelist"
      print "      : --stopglobal - stops DDS and Nameservice"
      print "      : --stoplocal  - stops the engines in the tuple list enginelist"
      # D-Bus exceptions are mapped to Python exceptions
      #try:
      #    iface.RaiseException()
      #except dbus.DBusException, e:
      #    print str(e)

      # introspection is automatically supported
      #print remote_object.Introspect(dbus_interface="org.freedesktop.DBus.Introspectable")
    #loop = gobject.MainLoop()
    #loop.run()   
if __name__ == '__main__':
    main()
